@@ -55,7 +55,7 @@ static GFIFileData * gfi_read_ttf_file_data (const gchar *name);
 static gboolean gfi_test_file_changed (GPFileEntry *f);
 #endif
 
-static void gfi_scan_directory (const gchar *name);
+static void gfi_scan_directory (const gchar *name, gint depth);
 static void gfi_try_font_file (const gchar *name);
 static void gfi_sort_fonts (void);
 static void gfi_build_fonts (void);
@@ -77,6 +77,7 @@ static void gfi_write_alias (xmlNodePtr root, GFIFontAlias *a);
 static gboolean gfi_debug = FALSE;
 static gboolean gfi_clean = FALSE;
 static gboolean gfi_recursive = FALSE;
+static gint gfi_recursion_depth = 16;
 static gboolean gfi_defaultpaths = FALSE;
 static gchar *target = NULL;
 static void add_alias_file (poptContext ctx, enum poptCallbackReason reason, const struct poptOption *opt, const char *arg, void *data);
@@ -113,9 +114,11 @@ static const struct poptOption options[] = {
 	  N_("Do not try to read existing fontmaps"), NULL },
 	{ "recursive", 'r', POPT_ARG_NONE, &gfi_recursive, 0,
 	  N_("Search directories recursively"), NULL },
+	{ "depth", 0, POPT_ARG_INT, &gfi_recursion_depth, 0,
+	  N_("Maximum recursion depth"), N_("DEPTH") },
 	{ "target", 't', POPT_ARG_STRING, &target, 0,
 	  N_("Output file name"), NULL },
-	{ NULL, '\0', POPT_ARG_CALLBACK, &add_alias_file, 0 },
+	{ NULL, 0, POPT_ARG_CALLBACK, &add_alias_file, 0 },
 	{ "aliases", 'a', POPT_ARG_STRING, NULL, 0,
 	  N_("Load aliases from given file"), N_("PATH") },
 	{ "defaultpaths", 0, POPT_ARG_NONE, &gfi_defaultpaths, 0,
@@ -192,7 +195,7 @@ int main (int argc, const char ** argv)
 	if (gfi_defaultpaths) {
 		if (gfi_debug) fprintf (stderr, "Scanning default directories:\n");
 		for (i = 0; i < NUM_DEFAULT_PATHS; i++) {
-			gfi_scan_directory (default_paths[i]);
+			gfi_scan_directory (default_paths[i], gfi_recursion_depth);
 		}
 		if (gfi_debug) fprintf (stderr, "Done default directories\n");
 	}
@@ -200,7 +203,7 @@ int main (int argc, const char ** argv)
 	if (args && args[0]) {
 		if (gfi_debug) fprintf (stderr, "Scanning specified directories:\n");
 		for (i = 0; args && args[i]; i++) {
-			gfi_scan_directory (args[i]);
+			gfi_scan_directory (args[i], gfi_recursion_depth);
 		}
 		if (gfi_debug) fprintf (stderr, "Done directories\n");
 	}
@@ -339,12 +342,14 @@ gfi_sort_filenames (GSList **names)
 /* Scan directories for font files */
 
 static void
-gfi_scan_directory (const gchar *name)
+gfi_scan_directory (const gchar *name, gint depth)
 {
 	DIR * dir;
 	struct dirent * dent;
 	struct stat s;
 
+	if (depth < 0) return;
+
 	dir = opendir (name);
 
 	if (dir) {
@@ -355,7 +360,7 @@ gfi_scan_directory (const gchar *name)
 			fn = g_strdup_printf ("%s/%s", name, dent->d_name);
 			if (!stat (fn, &s)) {
 				if (gfi_recursive && S_ISDIR (s.st_mode)) {
-					gfi_scan_directory (fn);
+					gfi_scan_directory (fn, depth - 1);
 				} else if (S_ISREG (s.st_mode)) {
 					gfi_try_font_file (fn);
 				}
@@ -909,264 +914,6 @@ gfi_write_alias (xmlNodePtr root, GFIFontAlias *a)
 	if (a->hideoriginal) xmlSetProp (n, "hideoriginal", "true");
 }
 
-#if 0
-static gchar *
-gfi_get_species_name (const gchar * fullname, const gchar * familyname)
-{
-	gchar * p;
-
-	p = strstr (fullname, familyname);
-
-	if (!p) return g_strdup ("Normal");
-
-	p = p + strlen (familyname);
-
-	while (*p && (*p < 'A')) p++;
-
-	if (!*p) return g_strdup ("Normal");
-
-	return g_strdup (p);
-}
-#endif
-
-#if 0
-static void
-add_assignment (poptContext ctx, enum poptCallbackReason reason, const struct poptOption *opt, const char *arg, void *data)
-{
-	struct stat s;
-	char *comma;
-
-	comma = strchr (arg, ',');
-
-	if (comma != NULL) {
-		*comma = '\0';
-		if ((stat (comma + 1, &s) == 0) && (S_ISDIR (s.st_mode))) {
-			gchar *key, *path;
-			GSList *alist;
-
-			key = g_strdup (arg);
-			path = g_strdup (comma + 1);
-			if (gfi_debug) fprintf (stderr, "Assigned %s : %s\n", key, path);
-
-			if (assignment_dict == NULL) assignment_dict = g_hash_table_new (g_str_hash, g_str_equal);
-
-			alist = g_hash_table_lookup (assignment_dict, key);
-			alist = g_slist_append (alist, path);
-			g_hash_table_insert (assignment_dict, key, alist);
-		}
-	}
-}
-#endif
-
-#if 0
-/* Scan those infamous .font files */
-
-static void
-gfi_scan_srcpaths (GSList * paths)
-{
-	GSList * l;
-
-	for (l = paths; l != NULL; l = l->next) {
-		DIR *dir;
-		struct dirent *direntry;
-		gchar * path;
-
-		path = (gchar *) l->data;
-
-		dir = opendir (path);
-		while ((direntry = readdir (dir))) {
-			gint len;
-			len = strlen (direntry->d_name);
-			if ((len > 5) && (!strcmp (direntry->d_name + len - 5, ".font"))) {
-				gchar *fn;
-				fn = g_concat_dir_and_file (path, direntry->d_name);
-				gfi_try_srcfile (fn);
-				g_free (fn);
-			}
-		}
-		closedir (dir);
-	}
-}
-
-/* Try to load .font file and split it into components */
-
-static void
-gfi_try_srcfile (const gchar *name)
-{
-	xmlDocPtr doc;
-	xmlNodePtr root;
-
-	if (gfi_debug) fprintf (stderr, "Trying srcfile %s\n", name);
-
-	doc = xmlParseFile (name);
-	if (doc) {
-		root = xmlDocGetRootElement (doc);
-		if (!strcmp (root->name, "font")) {
-			/* Single font entry */
-			gfi_try_srcfont (root);
-		} else if (!strcmp (root->name, "fontfile")) {
-			xmlNodePtr child;
-			/* List of font entries */
-			for (child = root->xmlChildrenNode; child != NULL; child = child->next) {
-				if (!strcmp (child->name, "font")) {
-					gfi_try_srcfont (child);
-				}
-			}
-		}
-	}
-}
-
-static void
-gfi_try_srcfont (xmlNodePtr node)
-{
-	char *format, *metrics, *glyphs, *alias;
-
-	format = xmlGetProp (node, "format");
-	metrics = xmlGetProp (node, "metrics");
-	glyphs = xmlGetProp (node, "glyphs");
-	alias = xmlGetProp (node, "alias");
-
-	/* Reasonable entry */
-	if (format && !strcmp (format, "type1") && metrics && glyphs) {
-		GFIFileData *afmd, *pfbd;
-
-		afmd = NULL;
-
-		if (metrics) {
-			if (*metrics == '*') {
-				gchar *slash;
-				GSList *path, *l;
-				/* find assignments etc. */
-				slash = strchr (metrics, '/');
-				if (slash) {
-					*slash = '\0';
-					slash++;
-					path = g_hash_table_lookup (assignment_dict, metrics + 1);
-					for (l = path; l != NULL; l = l->next) {
-						gchar *fn;
-						fn = g_concat_dir_and_file (l->data, slash);
-						afmd = g_hash_table_lookup (goodafm_dict, fn);
-						if (!afmd) afmd = gfi_read_afm_file_data (fn);
-						g_free (fn);
-						if (afmd) {
-							goodafm_list = g_slist_prepend (goodafm_list, afmd);
-							g_hash_table_insert (goodafm_dict, afmd->entry.name, afmd);
-							break;
-						}
-					}
-				}
-			} else if (*metrics == '/') {
-				afmd = g_hash_table_lookup (goodafm_dict, metrics);
-				if (!afmd) afmd = gfi_read_afm_file_data (metrics);
-				if (afmd) {
-					goodafm_list = g_slist_prepend (goodafm_list, afmd);
-					g_hash_table_insert (goodafm_dict, afmd->entry.name, afmd);
-				}
-			} else {
-				GSList *l;
-				for (l = afmpath_list; l != NULL; l = l->next) {
-					gchar *fn;
-					fn = g_concat_dir_and_file (l->data, metrics);
-					afmd = g_hash_table_lookup (goodafm_dict, fn);
-					if (!afmd) afmd = gfi_read_afm_file_data (fn);
-					g_free (fn);
-					if (afmd) {
-						goodafm_list = g_slist_prepend (goodafm_list, afmd);
-						g_hash_table_insert (goodafm_dict, afmd->entry.name, afmd);
-						break;
-					}
-				}
-			}
-		}
-
-		pfbd = NULL;
-
-		if (glyphs) {
-			if (*glyphs == '*') {
-				gchar *slash;
-				GSList *path, *l;
-				/* find assignments etc. */
-				slash = strchr (glyphs, '/');
-				if (slash) {
-					*slash = '\0';
-					slash++;
-					path = g_hash_table_lookup (assignment_dict, glyphs + 1);
-					for (l = path; l != NULL; l = l->next) {
-						gchar *fn;
-						fn = g_concat_dir_and_file (l->data, slash);
-						pfbd = g_hash_table_lookup (goodpfb_dict, fn);
-						if (!pfbd) pfbd = gfi_read_pfb_file_data (fn);
-						g_free (fn);
-						if (pfbd) {
-							goodpfb_list = g_slist_prepend (goodpfb_list, pfbd);
-							g_hash_table_insert (goodpfb_dict, pfbd->entry.name, pfbd);
-							break;
-						}
-					}
-				}
-			} else if (*glyphs == '/') {
-				pfbd = gfi_read_pfb_file_data (glyphs);
-				if (!pfbd) pfbd = gfi_read_pfb_file_data (glyphs);
-				if (pfbd) {
-					goodpfb_list = g_slist_prepend (goodpfb_list, pfbd);
-					g_hash_table_insert (goodpfb_dict, pfbd->entry.name, pfbd);
-				}
-			} else {
-				GSList *l;
-				for (l = pfbpath_list; l != NULL; l = l->next) {
-					gchar *fn;
-					fn = g_concat_dir_and_file (l->data, glyphs);
-					pfbd = g_hash_table_lookup (goodpfb_dict, fn);
-					if (!pfbd) pfbd = gfi_read_pfb_file_data (fn);
-					g_free (fn);
-					if (pfbd) {
-						goodpfb_list = g_slist_prepend (goodpfb_list, pfbd);
-						g_hash_table_insert (goodpfb_dict, pfbd->entry.name, pfbd);
-						break;
-					}
-				}
-			}
-		}
-
-		if (afmd && pfbd && (alias != NULL)) {
-			GPFontEntryT1Alias *new;
-
-			/* fixme: should check name matching */
-			new = g_new (GPFontEntryT1Alias, 1);
-			new->t1.entry.type = GP_FONT_ENTRY_TYPE1_ALIAS;
-			new->t1.entry.refcount = 1;
-			new->t1.entry.face = NULL;
-			new->t1.entry.name = g_strdup (afmd->fontname);
-			new->t1.entry.version = g_strdup (afmd->version);
-			new->t1.entry.familyname = g_strdup (afmd->familyname);
-			new->t1.entry.speciesname = gfi_get_species_name (afmd->fontname, afmd->familyname);
-			new->t1.entry.psname = g_strdup (afmd->psname);
-			/* fixme: this is fake, but it will not be used anyways */
-			new->t1.entry.weight = g_strdup ("Book");
-			/* AFM */
-			new->t1.afm.name = g_strdup (afmd->entry.name);
-			new->t1.afm.size = afmd->entry.size;
-			new->t1.afm.mtime = afmd->entry.mtime;
-			/* PFB */
-			new->t1.pfb.name = g_strdup (pfbd->entry.name);
-			new->t1.pfb.size = pfbd->entry.size;
-			new->t1.pfb.mtime = pfbd->entry.mtime;
-			/* Misc */
-			/* fixme: */
-			new->t1.Weight = GNOME_FONT_BOOK;
-			new->t1.ItalicAngle = 0.0;
-			new->alias = g_strdup (alias);
-
-			type1alias_list = g_slist_prepend (type1alias_list, new);
-		}
-	}
-
-	if (format) xmlFree (format);
-	if (metrics) xmlFree (metrics);
-	if (glyphs) xmlFree (glyphs);
-	if (alias) xmlFree (alias);
-}
-#endif
 
 /*
  * Checks, whether given alias file exists, and adds it to list of files
@@ -1184,277 +931,3 @@ add_alias_file (poptContext ctx, enum poptCallbackReason reason, const struct po
 	}
 }
 
-#if 0
-/*
- * Process GPFontEntry
- *
- * Tests, whether both afm and pfb files are valid
- * Saves file entries to good{$filetype}list
- * If it is Type1Alias, save it to type1aliaslist
- *
- */
-
-static void
-gfi_verify_font_entry (GPFontEntry * e)
-{
-	GPFontEntryT1 * t1;
-	GPFontEntryT1Alias * t1a;
-	GPFontEntryTT * tt;
-
-	switch (e->type) {
-	case GP_FONT_ENTRY_TYPE1_ALIAS:
-		t1a = (GPFontEntryT1Alias *) e;
-		type1alias_list = g_slist_prepend (type1alias_list, e);
-	case GP_FONT_ENTRY_TYPE1:
-		t1 = (GPFontEntryT1 *) e;
-		gfi_verify_afm_file (e, &t1->afm);
-		gfi_verify_pfb_file (e, &t1->pfb);
-		break;
-	case GP_FONT_ENTRY_TRUETYPE:
-		tt = (GPFontEntryTT *) e;
-		gfi_verify_ttf_file (e, &tt->ttf);
-		break;
-	case GP_FONT_ENTRY_ALIAS:
-		alias_list = gfi_add_alias_from_map (alias_list, e);
-		break;
-	default:
-		g_assert_not_reached ();
-	}
-}
-
-/*
- * Verifies afm entry
- *
- * If good, create new FileData and save it to goodafm_list/goodafm_dict
- *
- */
-
-static void
-gfi_verify_afm_file (GPFontEntry * e, GPFileEntry * f)
-{
-	GFIFileData * fd;
-
-	/* Test, whether we are already verified and registered */
-	if (g_hash_table_lookup (goodafm_dict, f->name)) return;
-
-	if (!gfi_test_file_changed (f)) {
-		fd = g_new0 (GFIFileData, 1);
-		fd->type = GFI_FILE_AFM;
-		fd->entry.name = g_strdup (f->name);
-		fd->entry.size = f->size;
-		fd->entry.mtime = f->mtime;
-		fd->fontname = g_strdup (e->name);
-		fd->familyname = g_strdup (e->familyname);
-		fd->psname = g_strdup (e->psname);
-		fd->version = g_strdup (e->version);
-	} else {
-		fd = gfi_read_afm_file_data (f->name);
-	}
-
-	if (fd) {
-		goodafm_list = g_slist_prepend (goodafm_list, fd);
-		g_hash_table_insert (goodafm_dict, fd->entry.name, fd);
-	} else {
-#ifdef GFI_VERBOSE
-		if (gfi_debug) fprintf (stderr, "Not good: %s\n", f->name);
-#endif
-	}
-}
-
-static void
-gfi_verify_pfb_file (GPFontEntry * e, GPFileEntry * f)
-{
-	GFIFileData * fd;
-
-	/* Test, whether we are already verified and registered */
-	if (g_hash_table_lookup (goodpfb_dict, f->name)) return;
-
-	if (!gfi_test_file_changed (f)) {
-		fd = g_new0 (GFIFileData, 1);
-		fd->type = GFI_FILE_PFB;
-		fd->entry.name = g_strdup (f->name);
-		fd->entry.size = f->size;
-		fd->entry.mtime = f->mtime;
-		fd->fontname = g_strdup (e->name);
-		fd->familyname = g_strdup (e->familyname);
-		fd->psname = g_strdup (e->psname);
-		fd->version = g_strdup (e->version);
-	} else {
-		fd = gfi_read_pfb_file_data (f->name);
-	}
-
-	if (fd) {
-		goodpfb_list = g_slist_prepend (goodpfb_list, fd);
-		g_hash_table_insert (goodpfb_dict, fd->entry.name, fd);
-	} else {
-#ifdef GFI_VERBOSE
-		if (gfi_debug) fprintf (stderr, "Not good: %s\n", f->name);
-#endif
-	}
-}
-
-static void
-gfi_verify_ttf_file (GPFontEntry * e, GPFileEntry * f)
-{
-	GFIFileData * fd;
-
-	/* Test, whether we are already verified and registered */
-	if (g_hash_table_lookup (goodttf_dict, f->name)) return;
-
-	if (!gfi_test_file_changed (f)) {
-		fd = g_new0 (GFIFileData, 1);
-		fd->type = GFI_FILE_TTF;
-		fd->entry.name = g_strdup (f->name);
-		fd->entry.size = f->size;
-		fd->entry.mtime = f->mtime;
-		fd->fontname = g_strdup (e->name);
-		fd->familyname = g_strdup (e->familyname);
-		fd->psname = g_strdup (e->psname);
-		fd->version = g_strdup (e->version);
-	} else {
-		fd = gfi_read_ttf_file_data (f->name);
-	}
-
-	if (fd) {
-		goodttf_list = g_slist_prepend (goodttf_list, fd);
-		g_hash_table_insert (goodttf_dict, fd->entry.name, fd);
-	} else {
-#ifdef GFI_VERBOSE
-		if (gfi_debug) fprintf (stderr, "Not good: %s\n", f->name);
-#endif
-	}
-}
-
-#endif
-
-#if 0
-static void
-gfi_process_type1_aliases (void)
-{
-	while (type1alias_list) {
-		gfi_process_type1_alias ((GPFontEntryT1Alias *) type1alias_list->data);
-		type1alias_list = g_slist_remove (type1alias_list, type1alias_list->data);
-	}
-}
-
-static void
-gfi_process_type1_alias (GPFontEntryT1Alias * t1a)
-{
-	GPFontEntryT1Alias * new;
-	GFIFileData * afmdata, * pfbdata;
-	GFIFontData * fontdata;
-	gdouble afmversion, pfbversion;
-	GSList * l;
-	FILE * f;
-	int status;
-	Font_Info * fi;
-
-	/* Return if we are already registered */
-	/* fixme: We should test versions here */
-	if (g_hash_table_lookup (goodfont_dict, t1a->t1.entry.name)) return;
-
-	afmdata = g_hash_table_lookup (goodafm_dict, t1a->t1.afm.name);
-	/* If we do not have good afm, return */
-	if (!afmdata) return;
-
-	/* We should probably use assertion here */
-	fontdata = g_hash_table_lookup (font_dict, afmdata->fontname);
-	if (!fontdata) return;
-
-	/* fixme: mess with locale */
-	afmversion = atof (afmdata->version);
-
-	/* Search, whether we have same or better pfb */
-	for (l = fontdata->pfb_list; l != NULL; l = l->next) {
-		gdouble pfbversion;
-		pfbdata = (GFIFileData *) l->data;
-		pfbversion = atof (pfbdata->version);
-		/* If we have original pfb with same or higher version, we shouldn't use type1 alias */
-		if (pfbversion >= afmversion) return;
-	}
-
-	/* So there wasn't original pfb file */
-
-	pfbdata = g_hash_table_lookup (goodpfb_dict, t1a->t1.pfb.name);
-	if (!pfbdata) return;
-
-	/* Find pfb file with highest version */
-	/* We should probably use assertion here */
-	fontdata = g_hash_table_lookup (font_dict, pfbdata->fontname);
-	if (!fontdata) return;
-
-	/* fixme: mess with locale */
-	pfbversion = atof (pfbdata->version);
-
-	for (l = fontdata->pfb_list; l != NULL; l = l->next) {
-		GFIFileData * d;
-		gdouble v;
-		d = (GFIFileData *) l->data;
-		v = atof (d->version);
-		if (v > pfbversion) {
-			pfbversion = v;
-			pfbdata = d;
-		}
-	}
-
-	/* We have to read afm to get weight and italicangle */
-
-	fi = NULL;
-	f = fopen (afmdata->entry.name, "r");
-	/* This shouldn't happen */
-	if (!f) return;
-	status = parseFile (f, &fi, P_G);
-	fclose (f);
-	/* This shouldn't happen! */
-	if (status != AFM_ok) {
-		if (fi) parseFileFree (fi);
-		return;
-	}
-
-	/* Now we should have everything we need */
-
-	new = g_new (GPFontEntryT1Alias, 1);
-	new->t1.entry.type = GP_FONT_ENTRY_TYPE1_ALIAS;
-	new->t1.entry.refcount = 1;
-	new->t1.entry.face = NULL;
-	new->t1.entry.name = g_strdup (afmdata->fontname);
-	new->t1.entry.version = g_strdup (afmdata->version);
-	new->t1.entry.familyname = g_strdup (t1a->t1.entry.familyname);
-	new->t1.entry.speciesname = g_strdup (t1a->t1.entry.speciesname);
-	new->t1.entry.psname = g_strdup (t1a->t1.entry.psname);
-	new->t1.entry.weight = g_strdup (fi->gfi->weight);
-	/* AFM */
-	new->t1.afm.name = g_strdup (afmdata->entry.name);
-	new->t1.afm.size = afmdata->entry.size;
-	new->t1.afm.mtime = afmdata->entry.mtime;
-	/* PFB */
-	new->t1.pfb.name = g_strdup (pfbdata->entry.name);
-	new->t1.pfb.size = pfbdata->entry.size;
-	new->t1.pfb.mtime = pfbdata->entry.mtime;
-	/* Misc */
-	new->t1.Weight = t1a->t1.Weight;
-	new->t1.ItalicAngle = fi->gfi->italicAngle;
-	new->alias = g_strdup (pfbdata->psname);
-
-#if 0
-	g_print ("<font format=\"alias\" "
-		 "name=\"%s\" familyname=\"%s\" speciesname=\"%s\" psname=\"%s\" "
-		 "original=\"%s\" hideoriginal=\"true\"/>\n",
-		 afmdata->fontname,
-		 t1a->t1.entry.familyname,
-		 t1a->t1.entry.speciesname,
-		 t1a->t1.entry.psname,
-		 pfbdata->fontname);
-#endif
-
-	/* Release AFM info */
-	parseFileFree (fi);
-
-	/* Register it */
-
-	if (gfi_debug) fprintf (stderr, "Registered Type1 alias: %s\n", new->t1.entry.name);
-
-	goodfont_list = g_slist_prepend (goodfont_list, new);
-	g_hash_table_insert (goodfont_dict, new->t1.entry.name, new);
-}
-#endif
