@@ -546,7 +546,7 @@ gnome_font_face_find_closest_from_weight_slant (const guchar *family, GnomeFontW
 	}
 
 	if (!face && map->fonts) {
-		face = gnome_font_face_find (((GPFontEntry *) map->fonts)->name);
+		face = gnome_font_face_find (((GPFontEntry *) map->fonts->data)->name);
 	}
 
 	gp_fontmap_release (map);
@@ -633,7 +633,7 @@ gnome_font_style_list (const guchar *family)
 void
 gnome_font_style_list_free (GList *styles)
 {
-	return gnome_font_family_style_list_free (styles);
+	gnome_font_family_style_list_free (styles);
 }
 
 GnomeFont *
