@@ -637,7 +637,7 @@ gnome_font_get_closest_pango_font (const GnomeFont *font, PangoFontMap *map, gdo
 	desc = gnome_font_get_pango_description (font, dpi);
 	g_return_val_if_fail (desc != NULL, NULL);
 
-	pfont = pango_font_map_load_font (map, desc);
+	pfont = pango_font_map_load_font (map, NULL, desc);
 
 	pango_font_description_free (desc);
 
