@@ -238,6 +238,7 @@ gnome_print_ps2_construct (GnomePrintContext *ctx)
 	GnomePrintPS2 *ps2;
 	gchar *tmp;
 	gint ret, fd;
+	const GnomePrintUnit *unit;
 
 	ps2 = GNOME_PRINT_PS2 (ctx);
 
@@ -260,8 +261,13 @@ gnome_print_ps2_construct (GnomePrintContext *ctx)
 	ps2->bbox.y0 = 0.0;
 	ps2->bbox.x1 = 21.0 * (72.0 / 2.54);
 	ps2->bbox.y1 = 29.7 * (72.0 / 2.54);
-	gpa_node_get_double_path_value (ctx->config, GNOME_PRINT_KEY_PAPER_WIDTH, &ps2->bbox.x1);
-	gpa_node_get_double_path_value (ctx->config, GNOME_PRINT_KEY_PAPER_HEIGHT, &ps2->bbox.y1);
+	/* fixme: We should not query physical, but logical page here */
+	if (gnome_print_config_get_length (ctx->config, GNOME_PRINT_KEY_PAPER_WIDTH, &ps2->bbox.x1, &unit)) {
+		gnome_print_convert_distance (&ps2->bbox.x1, unit, GNOME_PRINT_PS_UNIT);
+	}
+	if (gnome_print_config_get_length (ctx->config, GNOME_PRINT_KEY_PAPER_HEIGHT, &ps2->bbox.y1, &unit)) {
+		gnome_print_convert_distance (&ps2->bbox.y1, unit, GNOME_PRINT_PS_UNIT);
+	}
 
 	return GNOME_PRINT_OK;
 }
@@ -796,13 +802,12 @@ gp_ps2_set_font_private (GnomePrintPS2 *ps2, const GnomeFont *font)
 }
 
 GnomePrintContext *
-gnome_print_ps2_new (GPANode *config)
+gnome_print_ps2_new (GnomePrintConfig *config)
 {
 	GnomePrintContext *ctx;
 	gint ret;
 
 	g_return_val_if_fail (config != NULL, NULL);
-	g_return_val_if_fail (GPA_IS_NODE (config), NULL);
 
 	ctx = g_object_new (GNOME_TYPE_PRINT_PS2, NULL);
 
