@@ -16,8 +16,8 @@
 #include <sys/stat.h>
 #include <stdlib.h>
 #include <string.h>
-#include <libxml/parser.h>
 #include <libxml/xmlmemory.h>
+#include <libxml/parser.h>
 #include "gp-fontmap.h"
 
 static GPFontMap * gp_fontmap_load (void);
@@ -56,13 +56,13 @@ gp_fontmap_get (void)
 	static GPFontMap * map = NULL;
 
 	if (map) {
-		if (gp_fm_is_changed (map)) {
 #if 0
+		if (gp_fm_is_changed (map)) {
 			g_print ("Fontmap is changed, reloading\n");
-#endif
 			gp_fontmap_release (map);
 			map = NULL;
 		}
+#endif
 	}
 
 	if (!map) map = gp_fontmap_load ();
@@ -696,7 +696,7 @@ gp_fm_load_font_2_0_alias (GPFontMap *map, xmlNodePtr node)
 			if (!strcmp (child->name, "file")) {
 				xmlChar * type;
 				type = xmlGetProp (child, "type");
-				if (!strcmp (type, "pfb")) {
+				if (type && !strcmp (type, "pfb")) {
 					t1->pfb.name = gp_xmlGetPropString (child, "path");
 					t = xmlGetProp (child, "size");
 					if (t) t1->pfb.size = atoi (t);
@@ -705,7 +705,7 @@ gp_fm_load_font_2_0_alias (GPFontMap *map, xmlNodePtr node)
 					if (t) t1->pfb.mtime = atoi (t);
 					if (t) xmlFree (t);
 				}
-				xmlFree (type);
+				if (type) xmlFree (type);
 			}
 			if (t1->pfb.name) break;
 		}
@@ -824,7 +824,7 @@ gp_fm_load_font_3_0 (GPFontMap * map, xmlNodePtr node, gboolean type1alias)
 		if (!strcmp (child->name, "file")) {
 			xmlChar * type;
 			type = xmlGetProp (child, "type");
-			if (!type1alias && !strcmp (type, "afm")) {
+			if (!type1alias && type && !strcmp (type, "afm")) {
 				t1->afm.name = gp_xmlGetPropString (child, "path");
 				t = xmlGetProp (child, "size");
 				if (t) t1->afm.size = atoi (t);
@@ -832,7 +832,7 @@ gp_fm_load_font_3_0 (GPFontMap * map, xmlNodePtr node, gboolean type1alias)
 				t = xmlGetProp (child, "mtime");
 				if (t) t1->afm.mtime = atoi (t);
 				if (t) xmlFree (t);
-			} else if (!strcmp (type, "pfb")) {
+			} else if (type && !strcmp (type, "pfb")) {
 				t1->pfb.name = gp_xmlGetPropString (child, "path");
 				t = xmlGetProp (child, "size");
 				if (t) t1->pfb.size = atoi (t);
@@ -841,7 +841,7 @@ gp_fm_load_font_3_0 (GPFontMap * map, xmlNodePtr node, gboolean type1alias)
 				if (t) t1->pfb.mtime = atoi (t);
 				if (t) xmlFree (t);
 			}
-			xmlFree (type);
+			if (type) xmlFree (type);
 		}
 		if (t1->afm.name && t1->pfb.name) break;
 		child = child->next;
@@ -920,7 +920,7 @@ gp_fm_load_font_3_0_tt (GPFontMap * map, xmlNodePtr node)
 		if (!strcmp (child->name, "file")) {
 			xmlChar * type;
 			type = xmlGetProp (child, "type");
-			if (!strcmp (type, "ttf")) {
+			if (type && !strcmp (type, "ttf")) {
 				tt->ttf.name = gp_xmlGetPropString (child, "path");
 				t = xmlGetProp (child, "size");
 				if (t) tt->ttf.size = atoi (t);
@@ -929,7 +929,7 @@ gp_fm_load_font_3_0_tt (GPFontMap * map, xmlNodePtr node)
 				if (t) tt->ttf.mtime = atoi (t);
 				if (t) xmlFree (t);
 			}
-			xmlFree (type);
+			if (type) xmlFree (type);
 		}
 		if (tt->ttf.name) break;
 		child = child->next;
