@@ -254,15 +254,8 @@ gpa_key_lookup (GPANode *node, const guchar *path)
 
 	key = GPA_KEY (node);
 
-	if (!strncmp (path, "Option", 6)) {
-		if (!path[6]) {
-			gpa_node_ref (GPA_NODE (key->option));
-			return GPA_NODE (key->option);
-		} else {
-			g_return_val_if_fail (path[6] == '.', NULL);
-			return gpa_node_lookup (GPA_NODE (key->option), path + 7);
-		}
-	}
+	child = NULL;
+	if (gpa_node_lookup_ref (&child, GPA_NODE (key->option), path, "Option")) return child;
 
 	dot = strchr (path, '.');
 	if (dot != NULL) {
@@ -329,9 +322,9 @@ gpa_key_new_from_option (GPANode *node)
 	key = NULL;
 
 	switch (option->type) {
-	case GPA_OPTION_NODE:
-	case GPA_OPTION_KEY:
-	case GPA_OPTION_STRING:
+	case GPA_OPTION_TYPE_NODE:
+	case GPA_OPTION_TYPE_KEY:
+	case GPA_OPTION_TYPE_STRING:
 		key = (GPAKey *) gpa_node_new (GPA_TYPE_KEY, node->id);
 		key->option = node;
 		gpa_node_ref (key->option);
@@ -353,7 +346,7 @@ gpa_key_new_from_option (GPANode *node)
 			key->children = kch;
 		}
 		break;
-	case GPA_OPTION_LIST:
+	case GPA_OPTION_TYPE_LIST:
 		key = (GPAKey *) gpa_node_new (GPA_TYPE_KEY, node->id);
 		key->option = node;
 		gpa_node_ref (key->option);
@@ -380,7 +373,7 @@ gpa_key_new_from_option (GPANode *node)
 			g_warning ("Default was not in list (memory leak)");
 		}
 		break;
-	case GPA_OPTION_ITEM:
+	case GPA_OPTION_TYPE_ITEM:
 		/* No key for item */
 		break;
 	default:
@@ -408,7 +401,7 @@ gpa_key_write (xmlDocPtr doc, GPANode *node)
 
 	option = GPA_KEY_OPTION (key);
 
-	if (option->type != GPA_OPTION_KEY) {
+	if (option->type != GPA_OPTION_TYPE_KEY) {
 		xmln = xmlNewDocNode (doc, NULL, "Key", NULL);
 		xmlSetProp (xmln, "Id", node->id);
 		if (key->value) xmlSetProp (xmln, "Value", key->value);
@@ -617,9 +610,9 @@ gpa_key_merge_from_option (GPAKey *key, GPAOption *option)
 	key->option = GPA_NODE (option);
 
 	switch (option->type) {
-	case GPA_OPTION_NODE:
-	case GPA_OPTION_KEY:
-	case GPA_OPTION_STRING:
+	case GPA_OPTION_TYPE_NODE:
+	case GPA_OPTION_TYPE_KEY:
+	case GPA_OPTION_TYPE_STRING:
 		if (key->value) {
 			g_free (key->value);
 			key->value = NULL;
@@ -627,7 +620,7 @@ gpa_key_merge_from_option (GPAKey *key, GPAOption *option)
 		if (option->value) key->value = g_strdup (option->value);
 		gpa_key_merge_children_from_option (key, option);
 		break;
-	case GPA_OPTION_LIST:
+	case GPA_OPTION_TYPE_LIST:
 		och = NULL;
 		if (key->value) och = gpa_option_get_child_by_id (option, key->value);
 		if (och) {
@@ -643,7 +636,7 @@ gpa_key_merge_from_option (GPAKey *key, GPAOption *option)
 			}
 		}
 		break;
-	case GPA_OPTION_ITEM:
+	case GPA_OPTION_TYPE_ITEM:
 		/* No key for item */
 		break;
 	default:
