@@ -25,6 +25,7 @@
 
 #include <string.h>
 #include <libxml/xmlmemory.h>
+#include "gpa-utils.h"
 #include "gpa-value.h"
 #include "gpa-option.h"
 
@@ -32,6 +33,7 @@ static void gpa_option_class_init (GPAOptionClass *klass);
 static void gpa_option_init (GPAOption *option);
 static void gpa_option_finalize (GObject *object);
 
+static GPANode *gpa_option_duplicate (GPANode *node);
 static gboolean gpa_option_verify (GPANode *node);
 static guchar *gpa_option_get_value (GPANode *node);
 static GPANode *gpa_option_get_child (GPANode *node, GPANode *ref);
@@ -44,6 +46,17 @@ static GPAOption *gpa_option_new_list_from_tree (xmlNodePtr tree, const guchar *
 static GPAOption *gpa_option_new_item_from_tree (xmlNodePtr tree, const guchar *id);
 static GPAOption *gpa_option_new_string_from_tree (xmlNodePtr tree, const guchar *id);
 
+#define GPA_NOT -1
+#define GPA_MAYBE 0
+#define GPA_MUST 1
+
+#define GPA_FORBIDDEN(v) ((v) < 0)
+#define GPA_ALLOWED(v) ((v) == 0)
+#define GPA_MUSTHAVE(v) ((v) > 0)
+#define GPA_UNAMBIGUOUS(v) ((v) != 0)
+
+static gboolean gpa_option_xml_check (xmlNodePtr node, gint def, gint val, gint name, gint children);
+
 static GPANodeClass *parent_class;
 
 GType
@@ -78,6 +91,7 @@ gpa_option_class_init (GPAOptionClass *klass)
 
 	object_class->finalize = gpa_option_finalize;
 
+	node_class->duplicate = gpa_option_duplicate;
 	node_class->verify = gpa_option_verify;
 	node_class->get_value = gpa_option_get_value;
 	node_class->get_child = gpa_option_get_child;
@@ -88,7 +102,7 @@ gpa_option_class_init (GPAOptionClass *klass)
 static void
 gpa_option_init (GPAOption *option)
 {
-	option->type = GPA_OPTION_NONE;
+	option->type = GPA_OPTION_TYPE_NONE;
 	option->children = NULL;
 	option->value = NULL;
 }
@@ -101,18 +115,11 @@ gpa_option_finalize (GObject *object)
 	option = GPA_OPTION (object);
 
 	if (option->name) {
-		option->name->parent = NULL;
-		gpa_node_unref (option->name);
-		option->name = NULL;
+		option->name = gpa_node_detach_unref (GPA_NODE (option), option->name);
 	}
 
 	while (option->children) {
-		GPANode *child;
-		child = option->children;
-		option->children = child->next;
-		child->parent = NULL;
-		child->next = NULL;
-		gpa_node_unref (child);
+		option->children = gpa_node_detach_unref_next (GPA_NODE (option), option->children);
 	}
 
 	if (option->value) {
@@ -123,6 +130,43 @@ gpa_option_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static GPANode *
+gpa_option_duplicate (GPANode *node)
+{
+	GPAOption *option, *new;
+	GPANode *child;
+	GSList *l;
+
+	option = GPA_OPTION (node);
+
+	new = (GPAOption *) gpa_node_new (GPA_TYPE_OPTION, node->id);
+	new->type = option->type;
+	if (option->name) {
+		new->name = gpa_node_attach (GPA_NODE (new), gpa_node_duplicate (option->name));
+	}
+	if (option->value) {
+		new->value = g_strdup (option->value);
+	}
+
+	l = NULL;
+	for (child = option->children; child != NULL; child = child->next) {
+		GPANode *newchild;
+		newchild = gpa_node_duplicate (child);
+		if (newchild) l = g_slist_prepend (l, newchild);
+	}
+
+	while (l) {
+		GPANode *newchild;
+		newchild = GPA_NODE (l->data);
+		l = g_slist_remove (l, newchild);
+		newchild->parent = GPA_NODE (new);
+		newchild->next = new->children;
+		new->children = newchild;
+	}
+
+	return GPA_NODE (new);
+}
+
 static gboolean
 gpa_option_verify (GPANode *node)
 {
@@ -135,7 +179,7 @@ gpa_option_verify (GPANode *node)
 	if (!node->id) return FALSE;
 
 	switch (option->type) {
-	case GPA_OPTION_NODE:
+	case GPA_OPTION_TYPE_NODE:
 		/* !Name, !Value, children */
 		if (option->name) return FALSE;
 		if (option->value) return FALSE;
@@ -145,17 +189,17 @@ gpa_option_verify (GPANode *node)
 			if (!gpa_node_verify (child)) return FALSE;
 		}
 		break;
-	case GPA_OPTION_KEY:
+	case GPA_OPTION_TYPE_KEY:
 		/* !Name, Value || children */
 		if (option->name) return FALSE;
 		if (!option->value && !option->children) return FALSE;
 		for (child = option->children; child != NULL; child = child->next) {
 			if (!GPA_IS_OPTION (child)) return FALSE;
-			if (GPA_OPTION (child)->type != GPA_OPTION_KEY) return FALSE;
+			if (GPA_OPTION (child)->type != GPA_OPTION_TYPE_KEY) return FALSE;
 			if (!gpa_node_verify (child)) return FALSE;
 		}
 		break;
-	case GPA_OPTION_LIST:
+	case GPA_OPTION_TYPE_LIST:
 		/* List should not have name */
 		if (option->name) return FALSE;
 		/* List has to have default value */
@@ -164,11 +208,11 @@ gpa_option_verify (GPANode *node)
 		if (!option->children) return FALSE;
 		for (child = option->children; child != NULL; child = child->next) {
 			if (!GPA_IS_OPTION (child)) return FALSE;
-			if (GPA_OPTION (child)->type != GPA_OPTION_ITEM) return FALSE;
+			if (GPA_OPTION (child)->type != GPA_OPTION_TYPE_ITEM) return FALSE;
 			if (!gpa_node_verify (child)) return FALSE;
 		}
 		break;
-	case GPA_OPTION_ITEM:
+	case GPA_OPTION_TYPE_ITEM:
 		/* Item has to have name */
 		if (!option->name) return FALSE;
 		if (!gpa_node_verify (option->name)) return FALSE;
@@ -180,7 +224,7 @@ gpa_option_verify (GPANode *node)
 			if (!gpa_node_verify (child)) return FALSE;
 		}
 		break;
-	case GPA_OPTION_STRING:
+	case GPA_OPTION_TYPE_STRING:
 		/* String should not have name */
 		if (option->name) return FALSE;
 		/* String should have value */
@@ -345,17 +389,8 @@ gpa_option_new_node_from_tree (xmlNodePtr tree, const guchar *id)
 	xmlNodePtr xmlc;
 	GSList *l;
 
-	xmlval = xmlGetProp (tree, "Default");
-	if (xmlval) {
-		g_warning ("Option should not have default");
-		xmlFree (xmlval);
-		return NULL;
-	}
-
-	xmlval = xmlGetProp (tree, "Value");
-	if (xmlval) {
-		g_warning ("Option should not have value");
-		xmlFree (xmlval);
+	if (!gpa_option_xml_check (tree, GPA_NOT, GPA_NOT, GPA_NOT, GPA_MUST)) {
+		g_warning ("Option node structure is not correct");
 		return NULL;
 	}
 
@@ -363,10 +398,7 @@ gpa_option_new_node_from_tree (xmlNodePtr tree, const guchar *id)
 
 	for (xmlc = tree->xmlChildrenNode; xmlc != NULL; xmlc = xmlc->next) {
 		if (xmlc->type == XML_ELEMENT_NODE) {
-			if (!strcmp (xmlc->name, "Name")) {
-				g_warning ("List node should not have name");
-				return NULL;
-			} else if (!strcmp (xmlc->name, "Option")) {
+			if (!strcmp (xmlc->name, "Option") || !strcmp (xmlc->name, "Key")) {
 				GPANode *cho;
 				cho = gpa_option_new_from_tree (xmlc);
 				if (cho) l = g_slist_prepend (l, cho);
@@ -382,7 +414,7 @@ gpa_option_new_node_from_tree (xmlNodePtr tree, const guchar *id)
 	}
 
 	option = (GPAOption *) gpa_node_new (GPA_TYPE_OPTION, id);
-	option->type = GPA_OPTION_NODE;
+	option->type = GPA_OPTION_TYPE_NODE;
 
 	while (l) {
 		GPANode *cho;
@@ -406,10 +438,8 @@ gpa_option_new_key_from_tree (xmlNodePtr tree, const guchar *id)
 	xmlNodePtr xmlc;
 	GSList *l;
 
-	xmlval = xmlGetProp (tree, "Default");
-	if (xmlval) {
-		g_warning ("Key should not have default");
-		xmlFree (xmlval);
+	if (!gpa_option_xml_check (tree, GPA_NOT, GPA_MAYBE, GPA_NOT, GPA_MAYBE)) {
+		g_warning ("Option key structure is not correct");
 		return NULL;
 	}
 
@@ -423,11 +453,7 @@ gpa_option_new_key_from_tree (xmlNodePtr tree, const guchar *id)
 
 	for (xmlc = tree->xmlChildrenNode; xmlc != NULL; xmlc = xmlc->next) {
 		if (xmlc->type == XML_ELEMENT_NODE) {
-			if (!strcmp (xmlc->name, "Name")) {
-				g_warning ("Key node should not have name");
-				if (xmlval) xmlFree (xmlval);
-				return NULL;
-			} else if (!strcmp (xmlc->name, "Key")) {
+			if (!strcmp (xmlc->name, "Key")) {
 				GPANode *cho;
 				cho = gpa_option_new_from_tree (xmlc);
 				if (cho) l = g_slist_prepend (l, cho);
@@ -443,7 +469,7 @@ gpa_option_new_key_from_tree (xmlNodePtr tree, const guchar *id)
 	}
 
 	option = (GPAOption *) gpa_node_new (GPA_TYPE_OPTION, id);
-	option->type = GPA_OPTION_KEY;
+	option->type = GPA_OPTION_TYPE_KEY;
 	if (xmlval) {
 		option->value = g_strdup (xmlval);
 		xmlFree (xmlval);
@@ -467,40 +493,49 @@ static GPAOption *
 gpa_option_new_list_from_tree (xmlNodePtr tree, const guchar *id)
 {
 	GPAOption *option;
-	xmlChar *xmlval, *xmldef;
+	xmlChar *xmldef, *xmlval;
 	xmlNodePtr xmlc;
 	GSList *l;
 	gboolean has_default;
 
-	xmlval = xmlGetProp (tree, "Value");
-	if (xmlval) {
-		g_warning ("List node should not have value");
-		xmlFree (xmlval);
+	if (!gpa_option_xml_check (tree, GPA_MUST, GPA_NOT, GPA_NOT, GPA_MUST)) {
+		g_warning ("Option list structure is not correct");
 		return NULL;
 	}
 
 	xmldef = xmlGetProp (tree, "Default");
-	if (!xmldef) {
-		g_warning ("List node should have default");
-		return NULL;
-	}
 
 	l = NULL;
 	has_default = FALSE;
 
 	for (xmlc = tree->xmlChildrenNode; xmlc != NULL; xmlc = xmlc->next) {
 		if (xmlc->type == XML_ELEMENT_NODE) {
-			if (!strcmp (xmlc->name, "Name")) {
-				g_warning ("List node should not have name");
-				xmlFree (xmldef);
-				return NULL;
-			} else if (!strcmp (xmlc->name, "Item")) {
+			if (!strcmp (xmlc->name, "Item")) {
 				GPANode *cho;
 				cho = gpa_option_new_from_tree (xmlc);
 				if (cho) {
 					l = g_slist_prepend (l, cho);
 					if (!strcmp (xmldef, cho->id)) has_default = TRUE;
 				}
+			} else if (!strcmp (xmlc->name, "Fill")) {
+				xmlval = xmlGetProp (xmlc, "Ref");
+				if (xmlval) {
+					GPANode *def, *ref;
+					def = gpa_defaults ();
+					ref = gpa_node_get_path_node (def, xmlval);
+					gpa_node_unref (def);
+					xmlFree (xmlval);
+					if (GPA_OPTION_IS_LIST (ref)) {
+						GPANode *lc;
+						for (lc = ((GPAOption *) ref)->children; lc != NULL; lc = lc->next) {
+							GPANode *new;
+							new = gpa_node_duplicate (lc);
+							l = g_slist_prepend (l, new);
+							if (!strcmp (xmldef, new->id)) has_default = TRUE;
+						}
+					}
+					if (ref) gpa_node_unref (ref);
+				}
 			} else {
 				g_warning ("Invalid list child in option tree %s", xmlc->name);
 			}
@@ -524,7 +559,7 @@ gpa_option_new_list_from_tree (xmlNodePtr tree, const guchar *id)
 	}
 
 	option = (GPAOption *) gpa_node_new (GPA_TYPE_OPTION, id);
-	option->type = GPA_OPTION_LIST;
+	option->type = GPA_OPTION_TYPE_LIST;
 	option->value = g_strdup (xmldef);
 	xmlFree (xmldef);
 
@@ -551,48 +586,31 @@ gpa_option_new_item_from_tree (xmlNodePtr tree, const guchar *id)
 	GPANode *name;
 	GSList *l;
 
-	xmlval = xmlGetProp (tree, "Value");
-	if (xmlval) {
-		g_warning ("Item node should not have value");
-		xmlFree (xmlval);
+	if (!gpa_option_xml_check (tree, GPA_NOT, GPA_NOT, GPA_MUST, GPA_MAYBE)) {
+		g_warning ("Option item structure is not correct");
 		return NULL;
 	}
 
-	xmlval = xmlGetProp (tree, "Default");
-	if (xmlval) {
-		g_warning ("Item node should not have default");
-		xmlFree (xmlval);
-		return NULL;
-	}
+	xmlval = gpa_xml_node_get_name (tree);
+	name = gpa_value_new ("Name", xmlval);
+	xmlFree (xmlval);
 
-	name = NULL;
 	l = NULL;
 
 	for (xmlc = tree->xmlChildrenNode; xmlc != NULL; xmlc = xmlc->next) {
 		if (xmlc->type == XML_ELEMENT_NODE) {
-			if (!strcmp (xmlc->name, "Name")) {
-				name = gpa_value_new_from_tree ("Name", xmlc);
-			} else if (!strcmp (xmlc->name, "Option") || !strcmp (xmlc->name, "Key")) {
+			if (!strcmp (xmlc->name, "Option") || !strcmp (xmlc->name, "Key")) {
 				GPANode *cho;
 				cho = gpa_option_new_from_tree (xmlc);
 				if (cho) l = g_slist_prepend (l, cho);
-			} else {
+			} else if (strcmp (xmlc->name, "Name")) {
 				g_warning ("Invalid tag in option tree %s", xmlc->name);
 			}
 		}
 	}
 
-	if (!name) {
-		g_warning ("Missing \"Name\" tag in item description");
-		while (l) {
-			gpa_node_unref (GPA_NODE (l->data));
-			l = g_slist_remove (l, l->data);
-		}
-		return NULL;
-	}
-
 	option = (GPAOption *) gpa_node_new (GPA_TYPE_OPTION, id);
-	option->type = GPA_OPTION_ITEM;
+	option->type = GPA_OPTION_TYPE_ITEM;
 	option->name = name;
 
 	while (l) {
@@ -607,30 +625,23 @@ gpa_option_new_item_from_tree (xmlNodePtr tree, const guchar *id)
 	return option;
 }
 
-/* !Name, value, !children */
+/* !Name, default, !children */
 
 static GPAOption *
 gpa_option_new_string_from_tree (xmlNodePtr tree, const guchar *id)
 {
 	GPAOption *option;
-	xmlChar *xmlval;
 	xmlChar *defval;
 
-	xmlval = xmlGetProp (tree, "Value");
-	if (xmlval) {
-		g_warning ("Item node should not have value");
-		xmlFree (xmlval);
+	if (!gpa_option_xml_check (tree, GPA_MUST, GPA_NOT, GPA_NOT, GPA_NOT)) {
+		g_warning ("Option string structure is not correct");
 		return NULL;
 	}
 
 	defval = xmlGetProp (tree, "Default");
-	if (!defval) {
-		g_warning ("String node should have default");
-		return NULL;
-	}
 
 	option = (GPAOption *) gpa_node_new (GPA_TYPE_OPTION, id);
-	option->type = GPA_OPTION_STRING;
+	option->type = GPA_OPTION_TYPE_STRING;
 	option->value = g_strdup (defval);
 	xmlFree (defval);
 
@@ -694,568 +705,265 @@ gpa_option_get_child_by_id (GPAOption *option, const guchar *id)
 	return NULL;
 }
 
-#if 0
-GpaOption*
-gpa_option_get_from_id (GList *option_list, const gchar *id)
+GPANode *
+gpa_option_node_new (const guchar *id)
 {
-	GpaOption *option;
-	GList *list;
+	GPAOptionNode *option;
 
-	list = option_list;
-	for (; list != NULL; list = list->next) {
-		option = list->data;
-		if (strcmp (GPA_ITEM (option)->id, id) == 0)
-			return option;
-	}
+	g_return_val_if_fail (id != NULL, NULL);
+	g_return_val_if_fail (*id != '\0', NULL);
 
-	return NULL;
+	option = (GPAOption *) gpa_node_new (GPA_TYPE_OPTION, id);
+	option->type = GPA_OPTION_TYPE_NODE;
+
+	return (GPANode *) option;
 }
 
-#if 0
-/* fixme: temporary hack */
-static GPAModel *
-gpa_option_model (GpaOption *option)
+GPANode *
+gpa_option_list_new (const guchar *id)
 {
-	if (option->parent->parent) {
-		return GPA_MODEL (GPA_NODE (option->parent->parent->parent)->parent->parent);
-	} else {
-		return GPA_MODEL (GPA_NODE (option->parent)->parent->parent);
-	}
-}
-#endif
+	GPAOptionNode *option;
 
-GpaOption *
-gpa_option_new_from_node (xmlNodePtr tree_,
-			  GpaOptions *parent,
-			  GList *loaded_option_list)
-{
-	GpaOption *option;
-	xmlNodePtr tree;
-	xmlNodePtr child;
-	gchar *name;
-	gchar *id;
+	g_return_val_if_fail (id != NULL, NULL);
+	g_return_val_if_fail (*id != '\0', NULL);
 
-	debug (FALSE, "");
+	option = (GPAOption *) gpa_node_new (GPA_TYPE_OPTION, id);
+	option->type = GPA_OPTION_TYPE_LIST;
 
-	tree = gpa_include_node (tree_);
-
-	g_return_val_if_fail (tree != NULL, NULL);
-	
-	if (!gpa_xml_node_verify (tree, GPA_TAG_OPTION))
-		return NULL;
-
-	/* Get the name & id */
-	name = gpa_xml_get_value_string_required (tree, GPA_TAG_NAME, NULL);
-	id   = gpa_xml_get_value_string_required (tree, GPA_TAG_ID, parent->name);
-
-	if (name == NULL)
-		return NULL;
-	if (id == NULL)
-		return NULL;
-	
-	/* Find if there is another "option" object with this ID" */
-	if (gpa_option_get_from_id (loaded_option_list, id) != NULL) {
-		gpa_error ("The option \"%s\" is duplicated, under the "
-			   "\"%s\" options.", id, GPA_ITEM (parent)->id);
-		return NULL;
-	}
-
-	option = gpa_option_new (name, id, parent);
-
-	g_free (name);
-	g_free (id);
-
-	/* Load the child options for this option */
-	option->children = NULL;
-	child = gpa_xml_search_child (tree, GPA_TAG_OPTIONS);
-	if (child != NULL) {
-		option->children = gpa_options_new_from_node (child, option);
-		if (option->children == NULL)
-			return NULL;
-	}
-
-	/* Load the values */
-	option->values = NULL;
-	child = gpa_xml_search_child (tree, GPA_TAG_OPTION_INFO);
-	if (child != NULL) {
-		option->values = gpa_xml_utils_new_hash_from_node (child,
-							       GPA_TAG_OPTION_INFO);
-		if (option->values == NULL) {
-			gpa_error ("The \"%s\" node not be loaded", option->name);
-			return NULL;
-		}
-	}
-
-	/* Load the constraints */
-	option->constraints = NULL;
-	child = gpa_xml_search_child (tree, GPA_TAG_CONSTRAINTS);
-	if (child != NULL)
-		g_warning ("Constraints are no longer children of Options");
-
-	child = gpa_xml_search_child (tree, GPA_TAG_DEFAULT);
-	option->def = (child != NULL);
-
-	return option;
+	return (GPANode *) option;
 }
 
-/* fixme: Temporary hack (Lauris) */
-GList *
-gpa_option_parse_default (GpaOption *option, GList *list)
+GPANode *
+gpa_option_item_new (const guchar *id, const guchar *name)
 {
-	g_assert (option != NULL);
-	g_assert (GPA_IS_OPTION (option));
+	GPAOptionNode *option;
+	GPANode *child;
 
-	if (option->def) {
-		gchar *path;
-		path = gpa_option_dup_path (option);
-		list = g_list_prepend (list, path);
-	}
+	g_return_val_if_fail (id != NULL, NULL);
+	g_return_val_if_fail (*id != '\0', NULL);
+	g_return_val_if_fail (name != NULL, NULL);
+	g_return_val_if_fail (*name != '\0', NULL);
 
-	if (option->children) {
-		list = gpa_options_parse_default (option->children, list);
-	}
+	child = gpa_value_new ("Name", name);
+	g_return_val_if_fail (child != NULL, NULL);
 
-	return list;
+	option = (GPAOption *) gpa_node_new (GPA_TYPE_OPTION, id);
+	option->type = GPA_OPTION_TYPE_ITEM;
+	option->name = gpa_node_attach (GPA_NODE (option), child);
+
+	return (GPANode *) option;
 }
 
-GpaOption *
-gpa_option_copy (GpaOption *option,
-		 GpaOptions *parent)
+GPANode *
+gpa_option_string_new (const guchar *id, const guchar *value)
 {
-	GpaOption *new_option;
+	GPAOptionNode *option;
 
-	new_option = gpa_option_new (option->name,
-				     GPA_ITEM (option)->id,
-				     parent);
+	g_return_val_if_fail (id != NULL, NULL);
+	g_return_val_if_fail (*id != '\0', NULL);
+	g_return_val_if_fail (value != NULL, NULL);
+	g_return_val_if_fail (*value != '\0', NULL);
 
-	new_option->def = option->def;
-	new_option->children    = gpa_options_copy (option->children);
-	new_option->values      = gpa_hash_copy (option->values);
-	new_option->constraints = gpa_constraints_copy (option->constraints);
+	option = (GPAOption *) gpa_node_new (GPA_TYPE_OPTION, id);
+	option->type = GPA_OPTION_TYPE_STRING;
+	option->value = g_strdup (value);
 
-	return new_option;
+	return (GPANode *) option;
 }
 
-xmlNodePtr
-gpa_option_write (XmlParseContext *context, GpaOption *option)
+GPANode *
+gpa_option_key_new (const guchar *id, const guchar *value)
 {
-	xmlNodePtr node;
-	xmlNodePtr children;
-	xmlNodePtr values = NULL;
-	
-	debug (FALSE, "");
+	GPAOptionNode *option;
 
-	g_return_val_if_fail (option != NULL,         NULL);
-	g_return_val_if_fail (option->values != NULL, NULL);
+	g_return_val_if_fail (id != NULL, NULL);
+	g_return_val_if_fail (*id != '\0', NULL);
+	g_return_val_if_fail (!value || *value != '\0', NULL);
 
-	/* Write the FileInfo */
-	node = xmlNewDocNode (context->doc, context->ns, GPA_TAG_OPTION, NULL);
+	option = (GPAOption *) gpa_node_new (GPA_TYPE_OPTION, id);
+	option->type = GPA_OPTION_TYPE_KEY;
+	if (value) option->value = g_strdup (value);
 
-	/* Scan the list of options and if option->ty */
-	xmlNewChild (node, context->ns, GPA_TAG_NAME, option->name);
-	xmlNewChild (node, context->ns, GPA_TAG_ID,   GPA_ITEM (option)->id);
-
-	/* Write the values */
-	values = gpa_hash_write (context, option->values, GPA_TAG_OPTION_INFO);
-	if (values != NULL)
-		xmlAddChild (node, values);
-
-
-#if 0 /* We write constraints under the model node, now ..*/
-/* Write constraints */
-	constraints = gpa_constraints_write (context, option);
-	if (constraints != NULL)
-		xmlAddChild (node, constraints);
-#endif	
-
-	/* There are no options */
-	if (option->children == NULL)
-		return node;
-
-	/* The optional children structure */
-	children = gpa_options_write (context, option->children);
-	if (children != NULL)
-		xmlAddChild (node, children);
-
-	debug (FALSE, "");
-	
-	return node;
+	return (GPANode *) option;
 }
 
-
 gboolean
-gpa_option_verify (GpaOption *option)
+gpa_option_node_append_child (GPAOptionNode *option, GPAOption *child)
 {
-	debug (FALSE, "");
+	GPANode *ref;
 
 	g_return_val_if_fail (option != NULL, FALSE);
+	g_return_val_if_fail (GPA_IS_OPTION (option), FALSE);
+	g_return_val_if_fail (option->type == GPA_OPTION_TYPE_NODE, FALSE);
+	g_return_val_if_fail (child != NULL, FALSE);
+	g_return_val_if_fail (GPA_IS_OPTION (child), FALSE);
+	g_return_val_if_fail (GPA_NODE_PARENT (child) == NULL, FALSE);
 
-	if (option->name == NULL) {
-		gpa_error ("The Option object did not contained a name");
-		return FALSE;
-	}
-	if (GPA_ITEM (option)->id == NULL) {
-		gpa_error ("The Option \"%s\"object did not contained a valid id",
-			   option->name);
-		return FALSE;
+	ref = option->children;
+	while (ref && ref->next) ref = ref->next;
+
+	if (!ref) {
+		option->children = gpa_node_attach_ref (GPA_NODE (option), GPA_NODE (child));
+	} else {
+		ref->next = gpa_node_attach_ref (GPA_NODE (option), GPA_NODE (child));
 	}
 
-	if ((option->content != GPA_CONTENT_GENERIC) &&
-	    (option->content != GPA_CONTENT_PAPER_SIZE) &&
-	    (option->content != GPA_CONTENT_PAPER_MEDIA) &&
-	    (option->content != GPA_CONTENT_PAPER_SOURCE) &&
-	    (option->content != GPA_CONTENT_PAPER_ORIENTATION) &&
-	    (option->content != GPA_CONTENT_RESOLUTION) &&
-	    (option->content != GPA_CONTENT_RESOLUTION_MODE) &&
-	    (option->content != GPA_CONTENT_ERROR)) {
-		gpa_error ("The Option \"%s\" object contained an invalid content id [%i]",
-			   option->name, option->content);
-		return FALSE;
-	}
-
-	if (option->constraints != NULL)
-		if (!gpa_constraints_verify (option))
-			return FALSE;
-
-	if (!GPA_IS_OPTIONS (option->parent)) {
-		gpa_error ("The option \"%s\" object did not cotained a reference to a parent",
-			   option->name);
-		return FALSE;
-	}
-
-	if (option->parent->options_type == GPA_OPTIONS_TYPE_BOOLEAN) {
-		if ( (strcmp (GPA_ITEM (option)->id, GPA_TAG_TRUE) != 0) &&
-		     (strcmp (GPA_ITEM (option)->id, GPA_TAG_FALSE) != 0)) {
-			gpa_error ("The boolean option \"%s\" (under \"%s\") contained "
-				   "an invalid id :\"%s\"\nExpected (%s/%s)",
-				   option->name, option->parent->name, GPA_ITEM (option)->id,
-				   GPA_TAG_TRUE, GPA_TAG_FALSE);
-			return FALSE;
-		}
-	}
-	    
-		
-#if 0	/* FIXME */
-	if (option->values != NULL)
-		if (!gpa_values_verify_option (option))
-			return FALSE;
-#endif	
+	gpa_node_request_modified (GPA_NODE (option));
 
 	return TRUE;
 }
 
-
 gboolean
-gpa_option_verify_with_settings (GpaOption *option,
-				 GpaSettings *settings)
+gpa_option_list_append_child (GPAOptionList *option, GPAOption *child)
 {
+	GPANode *ref;
 
-	debug (FALSE, "");
-
-	g_return_val_if_fail (GPA_IS_SETTINGS (settings), FALSE);
 	g_return_val_if_fail (option != NULL, FALSE);
-	
-	if (!gpa_option_verify (option))
-		return FALSE;
-		
-	if (option->constraints != NULL)
-		if (!gpa_constraints_verify_with_settings (option, settings))
-			return FALSE;
+	g_return_val_if_fail (GPA_IS_OPTION (option), FALSE);
+	g_return_val_if_fail (option->type == GPA_OPTION_TYPE_LIST, FALSE);
+	g_return_val_if_fail (child != NULL, FALSE);
+	g_return_val_if_fail (GPA_IS_OPTION (child), FALSE);
+	g_return_val_if_fail (GPA_NODE_PARENT (child) == NULL, FALSE);
+
+	ref = option->children;
+	while (ref && ref->next) ref = ref->next;
+
+	if (!ref) {
+		option->children = gpa_node_attach_ref (GPA_NODE (option), GPA_NODE (child));
+	} else {
+		ref->next = gpa_node_attach_ref (GPA_NODE (option), GPA_NODE (child));
+	}
+
+	if (!option->value) {
+		option->value = g_strdup (GPA_NODE_ID (child));
+	}
+
+	gpa_node_request_modified (GPA_NODE (option));
 
 	return TRUE;
 }
 
-
-GpaOption *
-gpa_option_get_from_printer_and_path  (const GPAPrinter *printer, const gchar *path_)
+gboolean
+gpa_option_item_append_child (GPAOptionItem *option, GPAOption *child)
 {
-	g_return_val_if_fail (path_ != NULL, NULL);
-	g_return_val_if_fail (GPA_IS_PRINTER (printer), NULL);
+	GPANode *ref;
 
-	return gpa_option_get_from_options_list_and_path
-		(printer->model->options_list, path_);
-}
+	g_return_val_if_fail (option != NULL, FALSE);
+	g_return_val_if_fail (GPA_IS_OPTION (option), FALSE);
+	g_return_val_if_fail (option->type == GPA_OPTION_TYPE_ITEM, FALSE);
+	g_return_val_if_fail (child != NULL, FALSE);
+	g_return_val_if_fail (GPA_IS_OPTION (child), FALSE);
+	g_return_val_if_fail (GPA_NODE_PARENT (child) == NULL, FALSE);
 
-GpaOption *
-gpa_option_get_from_model_and_path  (const GPAModel *model, const gchar *path_)
-{
-	g_return_val_if_fail (path_ != NULL, NULL);
-	g_return_val_if_fail (GPA_IS_MODEL (model), NULL);
+	ref = option->children;
+	while (ref && ref->next) ref = ref->next;
 
-	return gpa_option_get_from_options_list_and_path
-		(model->options_list, path_);
-}
-
-GpaOption *
-gpa_option_get_from_settings_and_path (const GpaSettings *settings, const gchar *path_)
-{
-#if 0
-	/* fixme: Lauris */
-	g_return_val_if_fail (path_ != NULL, NULL);
-	g_return_val_if_fail (GPA_IS_SETTINGS (settings), NULL);
-
-	return gpa_option_get_from_options_list_and_path
-		(settings->printer->model->options_list, path_);
-#endif
-	return NULL;
-}
-
-
-static GpaOption *
-gpa_option_get_from_options_and_path (GpaOptions *options, const gchar *path)
-{
-	GpaOption *option = NULL;
-	GList *list;
-	gchar *option_id = NULL;
-	gint pos;
-
-	pos = gpa_tu_search (path, GPA_PATH_DELIMITER, TRUE);
-	if (pos == -1) {
-		gpa_error ("Invalid path %s, does not contain an Option\n", path);
-		return NULL;
-	}
-
-	option_id = g_strndup (path, pos);
-	list = options->children;
-	for (; list != NULL; list = list->next) {
-		option = (GpaOption *)list->data;
-		if (strcmp (GPA_ITEM (option)->id, option_id) == 0)
-			break;
-	}
-
-	if (list == NULL) {
-		gpa_error ("Option not found from path \"%s\" in \"%s\"(2)",
-			   path, gpa_options_get_name (options));
-		g_free (option_id);
-		return NULL;
-	}
-
-	if (strlen(path + pos + 1) > 0) {
-		GpaOption *sub_option;
-		gchar *subpath;
-		subpath = g_strdup (path+pos+1+strlen(GPA_ITEM (option->children)->id)+1);
-		sub_option = gpa_option_get_from_options_and_path (option->children,
-								   subpath);
-		g_free (subpath);
-		g_free (option_id);
-		return sub_option;
-	}
-	
-	g_free (option_id);
-
-	return option;
-
-}
-
-GpaOption *
-gpa_option_get_from_options_list_and_path (GPAList *options_list, const gchar *path_)
-{
-	GpaOptions *options;
-	GPAItem *item;
-	GpaOption *option = NULL;
-	gchar *options_id = NULL;
-	gchar *path;
-	gchar *remainder;
-	gint pos;
-
-	g_return_val_if_fail (options_list != NULL, NULL);
-	g_return_val_if_fail (path_   != NULL, NULL);
-
-	path = g_strdup_printf ("%s" GPA_PATH_DELIMITER, path_);
-	pos = gpa_tu_search (path, GPA_PATH_DELIMITER, TRUE);
-	if (pos == -1) {
-		gpa_error ("Invalid path %s\n", path);
-		goto gpa_option_get_from_options_list_and_path_error;
-	}
-
-	options_id = g_strndup (path, pos);
-
-	options = NULL;
-	for (item = options_list->items; item != NULL; item = item->next) {
-		options = GPA_OPTIONS (item);
-		if (strcmp (GPA_ITEM (options)->id, options_id) == 0)
-			break;
-	}
-
-	if (item == NULL) {
-		gpa_error ("Options not found from path \"%s\" (1.531)[%s]", path, path_);
-		goto gpa_option_get_from_options_list_and_path_error;
-	}
-
-	remainder = g_strdup (path+pos+1);
-	option = gpa_option_get_from_options_and_path (options, remainder);
-	g_free (remainder);
-
-	g_free (options_id);
-	g_free (path);
-
-	return option;
-
-gpa_option_get_from_options_list_and_path_error:
-
-	if (options_id != NULL)
-		g_free (options_id);
-	g_free (path);
-	
-	return NULL;
-}
-
-gboolean 
-gpa_option_is_selected (const GpaSettings *settings,
-			const GpaOption *option_findme)
-{
-	GpaOption *option= NULL;
-	GList *list;
-	gchar *path;
-
-	debug (FALSE, "");
-	
-	g_return_val_if_fail (option_findme != NULL, FALSE);
-	g_return_val_if_fail (GPA_IS_SETTINGS (settings), FALSE);
-
-	list = settings->selected_options;
-	for (; list != NULL; list = list->next) {
-		path = (gchar *) list->data;
-		option = gpa_option_get_from_settings_and_path (settings, path);
-		if (option == NULL)
-			return FALSE;
-		if (option == option_findme)
-			return TRUE;
-	}
-
-	return FALSE;
-}
-
-gchar *
-gpa_option_dup_path (GpaOption *option)
-{
-	gchar *path;
-	
-	debug (FALSE, "");
-
-	g_return_val_if_fail (option != NULL, NULL);
-
-	if (option->parent->parent != NULL) {
-		if (option->parent->parent->parent->parent != NULL) {
-			gpa_error ("We only support 1st & 2nd level option");
-		}
-		path  = g_strdup_printf ("%s" GPA_PATH_DELIMITER
-					 "%s" GPA_PATH_DELIMITER
-					 "%s" GPA_PATH_DELIMITER
-					 "%s",
-					 GPA_ITEM (option->parent->parent->parent)->id,
-					 GPA_ITEM (option->parent->parent)->id,
-					 GPA_ITEM (option->parent)->id,
-					 GPA_ITEM (option)->id);
+	if (!ref) {
+		option->children = gpa_node_attach_ref (GPA_NODE (option), GPA_NODE (child));
 	} else {
-		path = g_strdup_printf ("%s" GPA_PATH_DELIMITER "%s", GPA_ITEM (option->parent)->id, GPA_ITEM (option)->id);
-	}
-		
-	return path;
-}
-
-
-gchar*
-gpa_option_value_dup (const GpaOption *option, const gchar *key)
-{
-	GHashTable *hash;
-	gchar *value;
-	
-	debug (FALSE, "");
-
-	g_return_val_if_fail (option != NULL, NULL);
-	g_return_val_if_fail (key != NULL,    NULL);
-
-	hash = option->values;
-
-	if (hash == NULL) {
-		gpa_error ("Hash value not found. Option:\"%s\" Key:\"%s\"",
-			   option->name,
-			   key);
-		return NULL;
+		ref->next = gpa_node_attach_ref (GPA_NODE (option), GPA_NODE (child));
 	}
 
-	value = g_hash_table_lookup (hash, key);
+	gpa_node_request_modified (GPA_NODE (option));
 
-	debug (FALSE, "end");
-
-	return g_strdup (value);
+	return TRUE;
 }
 
-
-void
-gpa_option_value_get_int (const GpaOption *option, const gchar *key, int *value)
+gboolean
+gpa_option_key_append_child (GPAOptionKey *option, GPAOption *child)
 {
-	gchar *str_value;
-	
-	debug (FALSE, "");
+	GPANode *ref;
 
-	str_value = gpa_option_value_dup (option, key);
+	g_return_val_if_fail (option != NULL, FALSE);
+	g_return_val_if_fail (GPA_IS_OPTION (option), FALSE);
+	g_return_val_if_fail (option->type == GPA_OPTION_TYPE_KEY, FALSE);
+	g_return_val_if_fail (child != NULL, FALSE);
+	g_return_val_if_fail (GPA_IS_OPTION (child), FALSE);
+	g_return_val_if_fail (GPA_NODE_PARENT (child) == NULL, FALSE);
 
-	if (str_value == NULL) 
-		return;
+	ref = option->children;
+	while (ref && ref->next) ref = ref->next;
 
-	*value = atoi (str_value);
+	if (!ref) {
+		option->children = gpa_node_attach_ref (GPA_NODE (option), GPA_NODE (child));
+	} else {
+		ref->next = gpa_node_attach_ref (GPA_NODE (option), GPA_NODE (child));
+	}
 
-	g_free (str_value);
+	gpa_node_request_modified (GPA_NODE (option));
+
+	return TRUE;
 }
-	
-void
-gpa_option_value_get_double (const GpaOption *option, const gchar *key, double *value)
+
+/* Helper for checking <Name> "default" and "value" elements */
+
+static gboolean
+gpa_option_xml_check (xmlNodePtr node, gint def, gint val, gint name, gint children)
 {
-	gchar *str_value;
-	
-	debug (FALSE, "");
+	xmlChar *xmlval;
 
-	str_value = gpa_option_value_dup (option, key);
+	if (GPA_UNAMBIGUOUS (def)) {
+		xmlval = xmlGetProp (node, "Default");
+		if (xmlval && GPA_FORBIDDEN (def)) {
+			g_warning ("Node does not have \"Default\" attribute");
+			xmlFree (xmlval);
+			return FALSE;
+		} else if (!xmlval && GPA_MUSTHAVE (def)) {
+			g_warning ("Node should not have \"Default\" attribute");
+			xmlFree (xmlval);
+			return FALSE;
+		}
+	}
 
-	if (str_value == NULL) 
-		return;
+	if (GPA_UNAMBIGUOUS (val)) {
+		xmlval = xmlGetProp (node, "Value");
+		if (xmlval && GPA_FORBIDDEN (val)) {
+			g_warning ("Node does not have \"Value\" attribute");
+			xmlFree (xmlval);
+			return FALSE;
+		} else if (!xmlval && GPA_MUSTHAVE (val)) {
+			g_warning ("Node should not have \"Value\" attribute");
+			xmlFree (xmlval);
+			return FALSE;
+		}
+	}
 
-	*value = atof (str_value);
+	if (GPA_UNAMBIGUOUS (name)) {
+		xmlval = gpa_xml_node_get_name (node);
+		if (xmlval && GPA_FORBIDDEN (name)) {
+			g_warning ("Node does not have <Name> element");
+			xmlFree (xmlval);
+			return FALSE;
+		} else if (!xmlval && GPA_MUSTHAVE (name)) {
+			g_warning ("Node should not have <Name> element");
+			xmlFree (xmlval);
+			return FALSE;
+		}
+	}
 
-	g_free (str_value);
+	if (GPA_UNAMBIGUOUS (children)) {
+		xmlNodePtr child;
+		gboolean haschild;
+		/* This is quick check, as we do not verify children */
+		haschild = FALSE;
+		for (child = node->xmlChildrenNode; child != NULL; child = child->next) {
+			if (!strcmp (child->name, "Option") ||
+			    !strcmp (child->name, "Item") ||
+			    !strcmp (child->name, "Key") ||
+			    !strcmp (child->name, "Fill")) {
+				if (GPA_FORBIDDEN (children)) {
+					g_warning ("Node should not have children");
+					return FALSE;
+				}
+				haschild = TRUE;
+			}
+		}
+		if (!haschild && GPA_MUSTHAVE (children)) {
+			g_warning ("Node must have children");
+			return FALSE;
+		}
+	}
+
+	return TRUE;
 }
 
-/* Access to the structure */
-gchar *
-gpa_option_dup_name (const GpaOption *option)
-{
-	g_return_val_if_fail (GPA_IS_OPTION (option), NULL);
-
-	return g_strdup (option->name);
-}
-
-const gchar *
-gpa_option_get_name (const GpaOption *option)
-{
-	g_return_val_if_fail (GPA_IS_OPTION (option), NULL);
-
-	return option->name;
-}
-
-const gchar *
-gpa_option_get_id (const GpaOption *option)
-{
-	g_return_val_if_fail (GPA_IS_OPTION (option), NULL);
-
-	return GPA_ITEM (option)->id;
-}
-
-
-const GpaOptions *
-gpa_option_get_parent (const GpaOption *option)
-{
-	g_return_val_if_fail (GPA_IS_OPTION (option), NULL);
-
-	return option->parent;
-}
-
-const GpaOptions *
-gpa_option_get_children (const GpaOption *option)
-{
-	g_return_val_if_fail (GPA_IS_OPTION (option), NULL);
-
-	return option->children;
-}
-#endif
