@@ -25,8 +25,8 @@
 
 #include <string.h>
 #include <dirent.h> /* For the DIR structure stuff */
-#include <xmlmemory.h>
-#include <parser.h>
+#include <libxml/xmlmemory.h>
+#include <libxml/parser.h>
 
 #include "gpa-utils.h"
 #include "gpa-value.h"
