@@ -25,7 +25,7 @@
 
 #include <string.h>
 #include <malloc.h>
-#include <xmlmemory.h>
+#include <libxml/xmlmemory.h>
 #include "gpa-utils.h"
 #include "gpa-value.h"
 #include "gpa-reference.h"
