@@ -26,6 +26,7 @@
 #include <string.h>
 #include <unistd.h>
 #include <time.h>
+#include <libxml/parser.h>
 #include "gpa-node-private.h"
 #include "gpa-utils.h"
 
@@ -173,6 +174,40 @@ gpa_node_lookup_ref (GPANode **child, GPANode *node, const guchar *path, const g
 	return FALSE;
 }
 
+/* XML helpers */
+
+xmlChar *
+gpa_xml_node_get_name (xmlNodePtr node)
+{
+	xmlNodePtr child;
+
+	g_return_val_if_fail (node != NULL, NULL);
+
+	for (child = node->xmlChildrenNode; child != NULL; child = child->next) {
+		if (!strcmp (child->name, "Name")) {
+			return xmlNodeGetContent (child);
+		}
+	}
+
+	return NULL;
+}
+
+xmlNodePtr
+gpa_xml_node_get_child (xmlNodePtr node, const guchar *name)
+{
+	xmlNodePtr child;
+
+	g_return_val_if_fail (node != NULL, NULL);
+	g_return_val_if_fail (name != NULL, NULL);
+
+	for (child = node->xmlChildrenNode; child != NULL; child = child->next) {
+		if (!strcmp (child->name, name)) return child;
+	}
+
+	return NULL;
+}
+
+
 #if 0
 static void
 gpa_hash_item_copy (gpointer key_in, gpointer value_in, gpointer data)
